﻿using System;
using System.Threading;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Articles.ThreadingTutorial.Sample1
{
    public partial class Sample1Form : Form
    {
        private bool m_isWorkerRunning;
        private int m_workerDuration;

        /// <summary>
        /// Creates a new Sample1Form object.
        /// </summary>
        public Sample1Form()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Updates the form.
        /// </summary>
        private void UpdateForm()
        {
            m_lblLoading.Visible = m_isWorkerRunning;
            m_barLoading.Visible = m_isWorkerRunning;
            m_txtDuration.Enabled = !m_isWorkerRunning;
            m_cmdStartWork.Enabled = !m_isWorkerRunning;
        }

        /// <summary>
        /// Standard load-event (Called in GUI-Thread).
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            UpdateForm();
        }

        /// <summary>
        /// Called when user wants to close the form (Called in GUI-Thread).
        /// </summary>
        private void OnCmdCloseClick(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Called when user wants to start a background job (Called in GUI-Thread).
        /// </summary>
        private void OnCmdStartWorkClick(object sender, EventArgs e)
        {
            if (!m_isWorkerRunning)
            {
                if (Int32.TryParse(m_txtDuration.Text, out m_workerDuration))
                {
                    m_isWorkerRunning = true;
                    m_backgroundWorker.RunWorkerAsync();
                }
            }

            UpdateForm();
        }

        /// <summary>
        /// Called when BackgroundWorker starts working (Called in Worker-Thread).
        /// </summary>
        private void OnBackgroundWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            //Simulate some work
            Thread.Sleep(m_workerDuration * 1000);
        }

        /// <summary>
        /// Called when BackgroundWorker has finished (Called in GUI-Thread).
        /// </summary>
        private void OnBackgroundWorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            m_workerDuration = 0;
            m_isWorkerRunning = false;

            UpdateForm();
        }
    }
}
