﻿using System;
using System.Threading;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Articles.ThreadingTutorial.Sample1
{
    /// <summary>
    /// Interaktionslogik für Sample1Window.xaml
    /// </summary>
    public partial class Sample1Window : Window
    {
        private BackgroundWorker m_backgroundWorker;
        private bool m_isWorkerRunning;
        private int m_workerDuration;

        /// <summary>
        /// Creates a new Sample1Window object.
        /// </summary>
        public Sample1Window()
        {
            InitializeComponent();

            //Initialize the BackgroundWorker object
            m_backgroundWorker = new BackgroundWorker();
            m_backgroundWorker.DoWork += new DoWorkEventHandler(OnBackgroundWorkerDoWork);
            m_backgroundWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(OnBackgroundWorkerRunWorkerCompleted);
        }

        /// <summary>
        /// Updates the Window object.
        /// </summary>
        private void UpdateWindow()
        {
            m_lblLoading.Visibility = m_isWorkerRunning ? Visibility.Visible : Visibility.Hidden;
            m_barLoading.Visibility = m_isWorkerRunning ? Visibility.Visible : Visibility.Hidden;
            m_txtDuration.IsEnabled = !m_isWorkerRunning;
            m_cmdStartWork.IsEnabled = !m_isWorkerRunning;
        }

        /// <summary>
        /// Called when user wants to close the dialog.
        /// </summary>
        private void OnCmdCloseClick(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Called when user wants to start the background work.
        /// </summary>
        private void OnCmdStartWorkClick(object sender, RoutedEventArgs e)
        {
            if (!m_isWorkerRunning)
            {
                if (Int32.TryParse(m_txtDuration.Text, out m_workerDuration))
                {
                    m_isWorkerRunning = true;
                    m_backgroundWorker.RunWorkerAsync(null);
                }
            }

            UpdateWindow();
        }

        /// <summary>
        /// Called when BackgroundWorker starts working (Called in Worker-Thread).
        /// </summary>
        private void OnBackgroundWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            //Simulate some work
            Thread.Sleep(m_workerDuration * 1000);
        }

        /// <summary>
        /// Called when BackgroundWorker has finished (Called in GUI-Thread).
        /// </summary>
        private void OnBackgroundWorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            m_workerDuration = 0;
            m_isWorkerRunning = false;

            UpdateWindow();
        }
    }
}
